import java.util.*;

import org.eclipse.swt.*;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

/**
 * @author Thomas Singer
 */
public class StyledTextResourceLeakTest {

	public static void main(String[] args) {
		final DeviceData data = new DeviceData();
		data.tracking = true;
		final Display display = new Display(data);

		final Shell shell = new Shell(display);
		shell.setLayout(new FillLayout());
		final Button button = new Button(shell, SWT.PUSH);
		button.setText("Click to show a dialog");
		button.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				final Set<Object> oldObjects = getObjects();
				showDialogShell(shell);
				final Set<Object> newObjects = getObjects();
				newObjects.removeAll(oldObjects);
				System.out.println("newObjects = " + newObjects.size());

				for (Object object : newObjects) {
					System.out.println(object.getClass());
				}
			}

			private Set<Object> getObjects() {
				final DeviceData data = display.getDeviceData();
				return new HashSet<>(Arrays.asList(data.objects));
			}
		});
		shell.setSize(400, 300);
		shell.open();

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		display.dispose();
	}

	private static void showDialogShell(Shell parent) {
		final Shell shell = new Shell(parent, SWT.DIALOG_TRIM);
		shell.setLayout(new FillLayout());
		final StyledText styledText = new StyledText(shell, SWT.MULTI | SWT.BORDER);
		styledText.setText("Hello world");
		shell.setSize(300, 200);

		shell.open();

		final Display display = parent.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}
}
